import { useRef, useEffect, useMemo } from 'react';
import {
  timeFormat,
  scaleTime,
  extent,
  histogram as bin,
  timeMonths,
  sum,
  scaleLinear,
  max,
  brushX,
  select,
  event,
} from 'd3';
import { AxisBottom } from './AxisBottom';
import { AxisLeft } from './AxisLeft';
import { Marks } from './Mark';

const width = 960;
const margin = { top: 0, right: 30, bottom: 20, left: 50 };
const xAxisLabelOffset = 50;
const yAxisLabelOffset = 35;
const xAxisTickFormat = timeFormat('%m/%d/%Y');
const xAxisLabel = 'Time';
const yValue = (d) => d['Total Dead and Missing'];
const yAxisLabel = 'Total Dead and Missing';

export const DateHistogram = ({
  data,
  height,
  setBrushExtent,
  xValue,
}) => {
  const brushRef = useRef();

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const xScale = useMemo(
    () =>
      scaleTime()
        .domain(extent(data, xValue))
        .range([0, innerWidth])
        .nice(),
    [data, yValue, xValue, innerWidth]
  );

  const binnedData = useMemo(() => {
    const [start, stop] = xScale.domain();
    return bin()
      .value(xValue)
      .domain(xScale.domain())
      .thresholds(timeMonths(start, stop))(data)
      .map((arr) => ({
        y: sum(arr, yValue),
        x0: arr.x0,
        x1: arr.x1,
      }));
  }, [xValue, xScale, data]);

  const yScale = useMemo(
    () =>
      scaleLinear()
        .domain([0, max(binnedData, (d) => d.y)])
        .range([innerHeight, 0])
        .nice(),
    [binnedData, innerHeight]
  );

  useEffect(() => {
    const brush = brushX().extent([
      [0, 0],
      [innerWidth, innerHeight],
    ]);

    brush(select(brushRef.current));
    brush.on('brush end', () => {
      setBrushExtent(
        event.selection &&
          event.selection.map(xScale.invert)
      );
    });
  }, [innerWidth, innerHeight]);

  return (
    <>
      <rect
        width={width}
        height={height}
        fill="white"
        opacity="0.9"
      />
      <g
        transform={`translate(${margin.left},${margin.top})`}
      >
        <AxisBottom
          xScale={xScale}
          innerHeight={innerHeight}
          tickFormat={xAxisTickFormat}
          tickOffset={8}
        />
        <text
          className="axis-label"
          textAnchor="middle"
          transform={`translate(${-yAxisLabelOffset},${
            innerHeight / 2
          }) rotate(90)`}
        >
          {yAxisLabel}
        </text>
        <AxisLeft
          yScale={yScale}
          innerWidth={innerWidth}
          tickOffset={8}
        />
        <text
          className="axis-label"
          x={innerWidth / 2}
          y={innerHeight + xAxisLabelOffset}
          textAnchor="middle"
        >
          {xAxisLabel}
        </text>
        <Marks
          innerHeight={innerHeight}
          binnedData={binnedData}
          xScale={xScale}
          yScale={yScale}
          tooltipFormat={(d) => d}
        />
        <g ref={brushRef} />
      </g>
    </>
  );
};
